#include <stdio.h>
#include "shmem.h"

#define CONNECT_ERROR  1
#define ERROR -1
#define OK 0

extern int  CommInit(int *datasock, int *ctrlsock);
extern void CommQuit(int datasock, int ctrlsock);
extern int    RecvMapBounds(int datasock, int ctrlsock,
                            int *bound_north, int *bound_south, 
                            int *bound_east, int *bound_west);
extern void   FindInitRowCol(int bound_north, int bound_south, 
                             int bound_east, int bound_west, int north, 
                             int east, int res, int *row, int *col);
extern int    SendInitCoords(int datasock, int ctrlsock,
                             int north, int east);
extern int    RecvUrlList(int datasock, int ctrlsock);
extern int    RecvInitSegments(int datasock, int ctrlsock);

/* globals for socket numbers */
/* static int datasock, ctrlsock; data and control socket descriptors */
extern FILE *logfile;

int SetupConn(int *datasock, int *ctrlsock, int *bound_north, 
              int *bound_south, int *bound_east, int *bound_west, 
              int *res, int *row, int *col) {

   int             north, east;
   FILE            *fdinit;

/* initialize communication channels */
   printf("calling comminit\n");
   fflush(stdout);
   if (CommInit(datasock, ctrlsock) == CONNECT_ERROR) {
      fprintf(logfile, "connect error\n");
      return(ERROR);
   }

/* open the input file for coordinates */
   if ((fdinit = fopen("init.dat", "r")) == NULL)  {
     printf("ERROR: Could not open input file %s\n", "init.dat");
     if(fdinit) fclose(fdinit);
     CommQuit(*datasock, *ctrlsock);
     return(ERROR);
   }

/* read init coordinates from input file */
   fscanf(fdinit, "%d %d\n", &north, &east);
   if ((*res = RecvMapBounds(*datasock, *ctrlsock, bound_north, 
                             bound_south, bound_east,
                             bound_west)) == ERROR)  {
                                     /* get the bounding NEWS coordinates of
                                        dataset from server */
     fprintf(logfile, "error in reading map bounds\n");
     return(ERROR);
   }
printf("got bounds: %d %d %d %d \n", *bound_north, *bound_south, *bound_east,
        *bound_west);
   FindInitRowCol(*bound_north, *bound_south, *bound_east, *bound_west, 
                  north, east, *res, row, col);  
printf("init row col: %d %d \n", *row, *col);
                              /* convert the init utm coords to row, 
                                 column cell values */
   if (SendInitCoords(*datasock, *ctrlsock, north, east) == ERROR) {
     fprintf(logfile, "error in writing row/col info\n");
     return(ERROR);
   }
printf("sent n, e: %d %d \n", north, east);

   if (RecvUrlList(*datasock, *ctrlsock) == ERROR) {  
                                 /* get the urls in the requester region */
     fprintf(logfile, "error in reading url list\n");
     return(ERROR);
   }

   if (RecvInitSegments(*datasock, *ctrlsock) == ERROR) {  
                                       /* get the 9 segments from server */
     fprintf(logfile, "error in reading initial map segments from server\n");
     return(ERROR);
   }
   
   return(OK);

 }
