#include "defs.h"

pixel **colors;

void process(char *filename, int *height, int *width, int *maxval,
	     pixel ***grid)
{
  int i, j, lineptr;
  FILE *from;
  char MagicNumber[3];
  char *line;
  int linelength;

  from = fopen(filename, "r");
  if ((strcmp(filename, "/projects/kyk02/terrain/data/NYS/datafiles/landsat.albany") == 0) ||  (strcmp(filename, "/export/projects/kyk02/terrain/data/NYS/datafiles/landsat.albany") == 0))
    {
      fscanf(from, "%d%d%d%d%d%d", width, width, width, width, width, height); 
      MagicNumber[0] = 'P';
      MagicNumber[1] = '6';
      *maxval = 255;
      printf ("Width is %d, height is %d\n", *width, *height);
    }
  else
    {
      fgets( MagicNumber, 3, from);
      fscanf(from, "%d", width);
      fscanf(from, "%d", height);
      fscanf(from, "%d\n", maxval);
    }
  *grid = malloc(*height * sizeof(pixel*));
  for(i=0; i<*height; i++)
    (*grid)[i] = malloc(*width * sizeof(pixel));
  linelength = *width*3;
  line = malloc(linelength * sizeof(char));
  for (i=0; i<*height; i++)
    {
      fread(line, sizeof(char), linelength, from);
      lineptr = 0;
      for (j=0; j<*width; j++)
	{
	  (*grid)[i][j].red = line[lineptr];
	  (*grid)[i][j].green = line[lineptr+1];
	  (*grid)[i][j].blue = line[lineptr+2]; 
	  lineptr += 3;
	}
    }
  fclose(from);
  free(line);
}

void cleanup(pixel **what, int height)
{
  int i;
  for (i=0; i<height; i++)
    free(what[i]);
  free(what);
}

void magnify(int height, int width)
{
  pixel **big;
  int i,j, oldheight, oldwidth;
  int oldheight2, oldwidth2;
  big = malloc(height*2 * sizeof(pixel*));
  for(i=0; i<height*2; i++)
    big[i] = malloc(width*2 * sizeof(pixel));

  /* interpolate to get points in new bigger pixel array */
  for (i=0; i<height*2; i+=2)
    for (j=0; j<width*2; j+=2)
      {
	oldheight = (int)i/2;
	oldwidth = (int)j/2;
	big[i][j].red = colors[oldheight][oldwidth].red;
	big[i][j].green = colors[oldheight][oldwidth].green;
	big[i][j].blue = colors[oldheight][oldwidth].blue;
      }
  for (i=0; i<height*2; i+=2)
    {
      for (j=1; j<width*2-1; j+=2)
	{
	  big[i][j].red = (int)(big[i][j-1].red + big[i][j+1].red)/2;
	  big[i][j].green = (int)(big[i][j-1].green + big[i][j+1].green)/2;
	  big[i][j].blue = (int)(big[i][j-1].blue + big[i][j+1].blue)/2;
	}
      if ((width*2) % 2 == 0)
	{
	  oldwidth = width*2 - 1;
	  oldwidth2 = oldwidth - 1;
	  big[i][oldwidth].red = big[i][oldwidth2].red;
	  big[i][oldwidth].green = big[i][oldwidth2].green;
	  big[i][oldwidth].blue = big[i][oldwidth2].blue;
	}
    }
  for (i=1; i<height*2-1; i+=2)
    {
      for (j=0; j<width*2; j++)
	{
	  big[i][j].red = (int)(big[i-1][j].red + big[i+1][j].red)/2;
	  big[i][j].green = (int)(big[i-1][j].green + big[i+1][j].green)/2;
	  big[i][j].blue = (int)(big[i-1][j].blue + big[i+1][j].blue)/2;
	}
    }
  if ((height*2) % 2 == 0)
    {
      oldheight = height*2-1;
      oldheight2 = oldheight-1;
      for (j=0; j<width*2; j++)
	{
	  big[oldheight][j].red = big[oldheight2][j].red;
	  big[oldheight][j].green = big[oldheight2][j].green;
	  big[oldheight][j].blue = big[oldheight2][j].blue;
	}
    }

  cleanup(colors, height);
  colors = big;
}


void dump(char *filename, int height, int width, int maxval, pixel **grid)
{
  int i,j, lineptr, linelength;
  FILE *to;
  char *line;
  to = fopen(filename, "w");
  linelength = width*3;
  line = malloc(linelength*sizeof(char));
  fprintf(to, "P6\n");
  fprintf(to, "%d %d\n", width, height);
  fprintf(to, "%d\n", maxval);
  for (i=0; i<height; i++)
    {
      lineptr = 0;
      for (j=0; j<width; j++)
	{
	/*  fwrite(&grid[i][j].red, 1, 1, to);
	  fwrite(&grid[i][j].green, 1, 1, to);
	  fwrite(&grid[i][j].blue, 1, 1, to); */
	  line[lineptr] = grid[i][j].red;
	  line[lineptr+1] = grid[i][j].green;
	  line[lineptr+2] = grid[i][j].blue;
	  lineptr += 3;
	}
      fwrite(line, sizeof(char), linelength, to);
    }
  free(line);
  fclose(to);
}

 
