#include "init.h"

#define CONNECT_ERROR  1

/* globals for socket numbers */
/* static int datasock, ctrlsock; data and control socket descriptors */

/* and for the spaceflyer */
char *resource;

/* ----------------------------------------------------------------------
Name: CommInit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------ */
int CommInit(int *datasock, int *ctrlsock)  {

   char            hostaddr[50];
   int             dataport, ctrlport;
   FILE            *rcfile;

   rcfile = fopen(".2Ddemorc", "r");
   printf("rcfile: %d\n",rcfile);
   fflush(stdout);
   fscanf(rcfile, "%s\n%d\n%d", hostaddr, &dataport, &ctrlport);
   printf("hostaddr: %s dataport: %d ctrlport: %d\n", hostaddr, dataport, ctrlport);
   fflush(stdout);
   /* open the data and control sockets */
   if ((*datasock = ClientInit(hostaddr, dataport)) == ERROR)
      return (CONNECT_ERROR);
   if ((*ctrlsock = ClientInit(hostaddr, ctrlport)) == ERROR) {
      close(*datasock);
      return (CONNECT_ERROR);
   }

   if (rcfile) fclose (rcfile);
   return (OK);
}



/* ----------------------------------------------------------------------
Name: int RecvMapBounds(int *news)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int RecvMapBounds(int datasock, int ctrlsock, int *bound_north, 
                  int *bound_south, int *bound_east, int *bound_west) {

   int  res;
   char buf[100];

   if (ReadBytes(datasock, buf, 100) != 100) {
     printf("ERROR: In reading map bounds from data channel\n");
     return (ERROR);
   }
   sscanf(buf, "%d %d %d %d", bound_north, bound_south, bound_east,
                              bound_west);
   if (ReadInt(datasock, &res, 1) != 1) {
     printf("ERROR: In reading data resolution from data channel\n");
     return (ERROR);
   }

   return (res);

}




/* ----------------------------------------------------------------------
Name: int RecvMapBounds(int *news)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void FindInitRowCol(int bound_north, int bound_south, int bound_east,
                   int bound_west, int north, int east,
                   int res, int *row, int *col) {

   int temp1, temp2, temp3, temp4;

   temp1 = (int)floor((double)((east - bound_west)/res));
   temp2 = (int)floor((double)((bound_north - north)/res));
   temp3 = (int)floor((double)(temp1/SEGMENT_WIDTH));
   temp4 = (int)floor((double)(temp2/SEGMENT_HEIGHT));
   *col = temp3 * SEGMENT_WIDTH;  /* north west corner of segment */
   *row = temp4 * SEGMENT_HEIGHT; 

   return ;

}





/* ----------------------------------------------------------------------
Name: int RecvMapBounds(int *news)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SendInitCoords(int datasock, int ctrlsock, int north, int east) {

  char buf[120];

  sprintf(buf, "%d %d\n", north, east);
  if (WriteBytes(datasock, buf, 120) != 120) {
      printf("ERROR: In writing init coords to data channel\n");
      return (ERROR);
  }
 
}





/* ----------------------------------------------------------------------
Name: int RecvMapBounds(int *news)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int RecvUrlList(int datasock, int ctrlsock) {

  int  flag = FALSE, check, count;
  FILE *fd;
  char buf[250];
 
  fd = fopen("loclist", "a");

  if (ReadInt(ctrlsock, &count, 1) != 1) {
    printf("ERROR: In reading url count from server\n");
    if (fd) fclose(fd);
    return (ERROR);
  }
  fprintf(fd, "%d\n", count);
  while (TRUE) {
    if (ReadBytes(ctrlsock, buf, 250) != 250) {
      printf("ERROR: In reading url from server\n");
      flag = TRUE;
      break;
    }
    sscanf(buf, "%d", &check);
    if (check == -1) break;      /* end of transmission */
    fprintf(fd, "%s\n", buf);
  }

  if (flag) return (ERROR);
  if (fd) fclose(fd);
  return(OK); 
}





/* ----------------------------------------------------------------------
Name: int RecvMapBounds(int *news)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int RecvInitSegments(int datasock, int ctrlsock) {

  int i = 0, j = 0;
  FILE *fd;
  char fname[80], tbuf[40];
  char *buf;
  int  size;

  size = SEGMENT_HEIGHT * SEGMENT_WIDTH * 3;
  buf = (char *) malloc(size);
  for (j = 0; j < 3; j++) {  /* nine segments to be received */
    for (i = 0; i < 3; i++) {  /* nine segments to be received */
      memset(fname, '\0', 80);
      strcpy(fname, "seg_0.");
      memset(tbuf, '\0', 40);
      sprintf(tbuf, "%d.%d.ppm", j, i);
      strcat(fname, tbuf);
      fd = fopen(fname, "w");
      fprintf(fd, "%s\n", "P6");
      fprintf(fd, "%d\n %d\n %d\n", SEGMENT_HEIGHT, SEGMENT_WIDTH, 255);
      memset(buf, '\0', size);
      if (ReadBytes(datasock, buf, size) != size) {
	    printf("ERROR: error in reading segment for i=%d and j=%d\n", i, j);
	    if (fd) fclose(fd);
	    return(ERROR);
      }
      fwrite (buf, size, 1, fd);
      if (fd) fclose(fd);
    }
  }

  if (buf) free(buf);

  return(OK);
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
void CommQuit(int datasock, int ctrlsock)
{

/* close all socket streams */
   close(datasock);
   close(ctrlsock);

   printf("Goodbye\n");

}
