#include <errno.h>
#include "inet.h"
#include "shmem.h"

char       err_msg[180]; /* global error message buffer */

/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int ShmAlloc(int key, int datasize) {

  int shmid;

  if ((shmid = shmget(key, datasize, IPC_CREAT | IPC_EXCL | 0666)) < 0) {
    strcpy(err_msg, "ERROR: can't get shared memory");
    perror(err_msg);
    return (ERROR);
  }

  return (shmid);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int ShmAtch(int key, int datasize) {

  int shmid;

  if ((shmid = shmget(key, datasize, 0)) < 0) {
    strcpy(err_msg, "ERROR: can't get shared memory segment");
    perror(err_msg);
    return (ERROR);
  }  

  return (shmid);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int ShmDtch(char *addr) {

  if (shmdt(addr) < 0) {
    strcpy(err_msg, "ERROR: can't detach shared memory segment");
    perror(err_msg);
    return (ERROR);
  }
  
  return (OK);
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int ShmRel(int shmid) {

  if (shmctl(shmid, IPC_RMID, (struct shmid_ds *) 0) < 0) {
    strcpy(err_msg, "ERROR: can't release shared memory");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemCreate(int key, int initval) {

  int        sem;
  semctl_arg arg;
  char       temp[80];
  
  arg.val = 0;

  if ((sem = semget(key, 1, IPC_CREAT | IPC_EXCL | 0666)) == ERROR) {
    strcpy(err_msg, "ERROR: can't create semaphore with key = ");
    sprintf(temp, "%d", key);
    strcat(err_msg, temp);
    perror(err_msg);
    return (ERROR);
  }

  /* initialize semaphore */
  arg.val = initval;
  if (semctl(sem, 0, SETVAL, arg) < 0) {
    strcpy(err_msg, "ERROR: can't SETVAL");
    perror(err_msg);
    SemRem(sem);
    return (ERROR);
  }

  return (sem);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemCreate2(key_t key, int initval1, int initval2) {

  int        sem;
  semctl_arg arg;
  char       temp[80];
  
  arg.val = 0;

  if ((sem = semget(key, 2, IPC_CREAT | IPC_EXCL | 0666)) == ERROR) {
    strcpy(err_msg, "ERROR: can't create semaphore with key = ");
    sprintf(temp, "%d", key);
    strcat(err_msg, temp);
    perror(err_msg);
    return (ERROR);
  }

  /* initialize semaphores */
  arg.val = initval1;
  if (semctl(sem, 0, SETVAL, arg) < 0) {
    strcpy(err_msg, "ERROR: can't SETVAL for semaphore #0");
    perror(err_msg);
    SemRem(sem);
    return (ERROR);
  }
  arg.val = initval2;
  if (semctl(sem, 1, SETVAL, arg) < 0) {
    strcpy(err_msg, "ERROR: can't SETVAL for semaphore #1");
    perror(err_msg);
    SemRem(sem);
    return (ERROR);
  }

  return (sem);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemValCheck2(int semid, int val0, int val1) {

  semctl_arg arg;
  
  arg.val = val0;
  if (semctl(semid, 0, GETVAL, arg) != val0) {
    strcpy(err_msg, "semaphore #0 not initialized");
    perror(err_msg);
    return (0);
  }
  arg.val = val1;
  if (semctl(semid, 1, GETVAL, arg) != val1) {
    strcpy(err_msg, "semaphore #1 not initialized");
    perror(err_msg);
    return (0);
  }

  return (1);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemRem(int sem) {

  semctl_arg arg;

  arg.val = 0;
  if (semctl(sem, 0, IPC_RMID, arg) < 0) {
    strcpy(err_msg, "ERROR: can't remove semaphore");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);

}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemWait(int sem) {

  if (semop(sem, &op_lock[0], 2) < 0) {
    strcpy(err_msg, "ERROR: can't semop on lock");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemSignal(int sem) {

  if (semop(sem, &op_unlock[0], 1) < 0) {
    strcpy(err_msg, "ERROR: can't semop on unlock\n");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemOpen(int key) {

  int  sem;
  char temp[80];

  if ((sem = semget(key, 0, 0)) == ERROR) {
    strcpy(err_msg, "ERROR: can't open semaphore with key = ");
    sprintf(temp, "%d", key);
    strcat(err_msg, temp);
    perror(err_msg);
    return (ERROR);
  }

  return (sem);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemCliWait(int sem) {

  if (semop(sem, &cli_lock[0], 2) < 0) {
    strcpy(err_msg, "ERROR: can't semop on client lock");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}




/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemCgiWait(int sem) {

  if (semop(sem, &cgi_lock[0], 2) < 0) {
    strcpy(err_msg, "ERROR: can't semop on cgi lock");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemDispWait(int sem) {

  if (semop(sem, &disp_lock[0], 2) < 0) {
    strcpy(err_msg, "ERROR: can't semop on displayer lock");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemCliSignal(int sem) {

  if (semop(sem, &cli_unlock[0], 1) < 0) {
    strcpy(err_msg, "ERROR: can't semop on client unlock");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}





/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemCgiSignal(int sem) {

  if (semop(sem, &cgi_unlock[0], 1) < 0) {
    strcpy(err_msg, "ERROR: can't semop on cgi unlock");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}






/* ----------------------------------------------------------------------
Name: CommQuit(int *datasock, int *ctrlsock)
In: None
Out: sockfd
Comments: This function connects the data and control sockets to the
          server and returns the datasockfd and ctrlsockfd
------------------------------------------------------------------------- */
int SemDispSignal(int sem) {

  if (semop(sem, &disp_unlock[0], 1) < 0) {
    strcpy(err_msg, "ERROR: can't semop on displayer unlock");
    perror(err_msg);
    return (ERROR);
  }

  return (OK);
}

