#include <sys/time.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/errno.h>
#include <string.h>
#ifdef RS6000
#include <rpc/types.h>
#endif
#include <rpc/types.h>
#include <rpc/xdr.h>

#include "inet.h"

#define QLEN    5
#define BUFSIZE    4096

/* Funcitons in this file */
int Server(char *, char *, char *, int);
int BindServer(char *, char *, char *, char *);
int AcceptClient(int );
int AcceptControl(int);
int Client(char *, char *, char *, int);
int Client2(char *, char *, char *, int);
int ClientInit(char *, int);
int ClientInit2(char *, int);
int SendImage(unsigned char *, int , int );
int ReadBytes(int , char *, int );
int ReadShort(int, short *, int);
int ReadInt(int, int *, int);
int ReadShort2Float(int, float *, int);
int ReadDouble(int, double *, int);
int WriteBytes(int , char *, int );
int WriteShort(int, short *, int);
int WriteInt(int, int *, int);
int WriteDouble(int, double *, int);

/* Globals - there's got to be a way around this! Think! */
static char *host, *serv, *proto;
static int port;
static int psock;               /* socket descriptor */
static struct sockaddr_in  serv_addr;




/*--------------------------------------------------------------------------
Name: int Server(char *hostname, char *service, char *protocol, char *channel)
In: None
Out: sockfd
Comments: This function binds the server data/control sockets and accepts
          connection to socket. Returns -1 in case connect fails.
--------------------------------------------------------------------------*/
int
Server(char *hostaddr, char *service, char *protocol, int port) {

  int  csock;           /* child socket */
  int  clilen;          /* client socket */
  struct sockaddr_in   serv_addr, cli_addr;

  psock=OpenSock(hostaddr, service, protocol, &serv_addr, port);
  printf("Binding server on port: %hu at IP address: %lu \n",
         serv_addr.sin_port, serv_addr.sin_addr); 
  if (bind(psock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
    printf("ERROR: Can't bind server\n");
    return(ERROR);
  }

  if (listen(psock, QLEN) < 0) {
    printf("ERROR: Can't listen\n");
    return(ERROR);
  }

  clilen = sizeof(struct sockaddr_in) ;
  csock=accept(psock, (struct sockaddr *)&cli_addr, &clilen);
  if (csock < 0) {
    printf("ERROR: Accept error\n");
    return(ERROR);
  }
  close(psock);
  return(csock);

}




/*--------------------------------------------------------------------------
Name: int BindServer(char *hostname, char *service, char *protocol, 
                     char *channel)
In: None
Out: sockfd
Comments: This function binds the server data/control sockets.
--------------------------------------------------------------------------*/
int BindServer(char *hostname, char *service, char *protocol, 
	       char *channel) {

  int  csock;           /* child socket */
  struct sockaddr_in   serv_addr;

  if (!strcmp(channel, "data"))
    port = SERV_DATA_PORT;
  else if (!strcmp(channel, "control"))
    port = SERV_CONTROL_PORT;

  psock=OpenSock(hostname, service, protocol, &serv_addr, port);
  printf("Binding server on port: %hu at IP address: %lu \n",
         serv_addr.sin_port, serv_addr.sin_addr); 
  if (bind(psock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
    printf("ERROR: Can't bind server\n");
    return(ERROR);
  }

  if (listen(psock, QLEN) < 0) {
    printf("ERROR: Can't listen\n");
    return(ERROR);
  }

  return(psock);

}




/*--------------------------------------------------------------------------
Name: int AcceptClient(int psock)
In: None
Out: sockfd
Comments: This function accepts connection from client. Using concurrent
          server model.
--------------------------------------------------------------------------*/
int AcceptClient(int psock) {

  int                  csock;
  int                  clilen;
  struct sockaddr_in   cli_addr;
  int                  childpid;

  clilen = sizeof(struct sockaddr_in) ;
  csock=accept(psock, (struct sockaddr *)&cli_addr, &clilen);
  if (csock < 0) {
    perror("ERROR: accept error:\n");
    return(ERROR);
  }

  return(csock);

}



/*--------------------------------------------------------------------------
Name: int AcceptControl(int psock)
In: None
Out: sockfd
Comments: This function accepts control connection from client.
--------------------------------------------------------------------------*/
int AcceptControl(int psock) {

  int csock;
  int clilen;
  struct sockaddr_in   cli_addr;

  clilen = sizeof(struct sockaddr_in) ;
  csock=accept(psock, (struct sockaddr *)&cli_addr, &clilen);
  if (csock < 0) {
    printf("ERROR: Control channel accept error\n");
    return(ERROR);
  }

  close (psock);
  return(csock);

}




/*--------------------------------------------------------------------------
Name: int Client(char *hostname, char *service, char *protocol
                 char *channel)
In: None
Out: sockfd
Comments: This function connects to the server data/control sockets. 
          Retries connection at every 1 sec. interval, timeout is (soft)set
          to 10 attempts.
          Returns -1 in case connect fails.
--------------------------------------------------------------------------*/
int Client(char *hostaddr, char *service, char *protocol, int port) { 

  struct sockaddr_in  serv_addr;
  int prev_hdl = 0;
  int oldmask;
  int count = 0;
  int connected = TRUE;

  psock=OpenSock(hostaddr, service, protocol, &serv_addr, port);
  count++ ;
  while(connect(psock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0){
    if (count == TIMEOUT_N) {
      printf("ERROR: Failed to connect to server: %s at port: %hu \
            after %d attempts \n", hostaddr, serv_addr.sin_port, count);
      connected = FALSE;
      break;
    }
    printf("Retrying connection \n");
    sleep(1);
    psock=OpenSock(hostaddr, service, protocol, &serv_addr, port);
    count++ ;
  }
  
  if (connected == FALSE)
    return(ERROR);
  return (psock);

}




/*--------------------------------------------------------------------------
Name: int Client(char *hostname, char *service, char *protocol
                 char *channel)
In: None
Out: sockfd
Comments: This function connects to the server data/control sockets.
          Retries connection at every 1 sec. interval, timeout is (soft)set
         to 10 attempts.
          Returns -1 in case connect fails.
--------------------------------------------------------------------------*/
int Client2(char *hostname, char *service, char *protocol, int conn_port) {

  struct sockaddr_in  serv_addr;
  int prev_hdl = 0;
  int oldmask;
  int count = 0;
  int connected = TRUE;

  strcpy((host=(char *)malloc(MAXHOSTNAMELEN + 1)), hostname);
  strcpy((serv=(char *)malloc(10)), service);
  strcpy((proto=(char *)malloc(10)), protocol);

  psock=OpenSock(hostname, service, protocol, &serv_addr, conn_port);
  count++ ;
  while(connect(psock, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0){
    if (count == TIMEOUT_N) {
      printf("ERROR: Failed to connect to server: %s at port: %hu \
            after %d attempts \n", hostname, serv_addr.sin_port, count);
      connected = FALSE;
      break;
    }
    printf("Retrying connection \n");
    sleep(1);
    psock=OpenSock(hostname, service, protocol, &serv_addr, conn_port);
    count++ ;
  }

  if (connected == FALSE)
    return(ERROR);
  return (psock);

}





/*--------------------------------------------------------------------------
Name: int ClientInit(char *hostname, char *channel)
In: None
Out: sockfd
Comments: This function connects to the server data/control sockets.
          Returns -1 in case connect fails.
--------------------------------------------------------------------------*/
int ClientInit(char *hostaddr, int port) {

   int sockfd;

   printf("Connecting %d port\n", port);
   if ((sockfd=Client(hostaddr, "tcp", "tcp", port)) == ERROR) {
     return(ERROR);
   }
   printf("%d port connection established\n", port);

   return (sockfd);

}




/*--------------------------------------------------------------------------
Name: int ClientInit(char *hostname, char *channel)
In: None
Out: sockfd
Comments: This function connects to the server data/control sockets.
          Returns -1 in case connect fails.
--------------------------------------------------------------------------*/
int ClientInit2(char *hostname, int conn_port) {

   int sockfd;

   printf("Connecting %d port\n", port);
   if ((sockfd=Client2(hostname, "tcp", "tcp", conn_port)) == ERROR) {
     printf("ERROR in connecting %d port\n", conn_port);
     return(ERROR);
   }
   printf("%d port connection established\n", conn_port);

   return (sockfd);

}






/*--------------------------------------------------------------------------
NOTE: Implementing blocking read. Require that message length from client match the size of view defined in view.h
--------------------------------------------------------------------------*/

/*
int
GetView(sockfd, view)

int sockfd;
char *view;

{
int nread, status = 1;
char *flag;

flag = (char *)malloc(100);
if((nread = readn(sockfd, view, VIEWLEN)) != VIEWLEN) {
  printf("ERROR server: readn error\n");
  exit (ERROR);
}
sscanf(view, "%s", flag);
if (strcmp(flag, "EOT") == 0)
  status = NULL;

return status;
}
*/





/*-----------------------------------------------------------------------
Function to send image to the front-end 
------------------------------------------------------------------------*/

int
SendImage(unsigned char *buf, int size, int sockfd)

{
  return (WriteBytes(sockfd, buf, size));
}




/*-----------------------------------------------------------------------
Name: ReadBytes(int sockfd, char *message, int size)
In: signature
Out: message from socket
Comments: Function to read message from a socket using XDR.
------------------------------------------------------------------------*/
int ReadBytes(int sockfd, char *message, int size) {

  XDR xdrs;
  char *buf;
  int nbytes;

  buf = (char *)malloc(size+16);
  xdrmem_create(&xdrs, buf, size+16, XDR_DECODE);
  if ((nbytes = readn(sockfd, buf, size)) != size)
    return (ERROR);
  else {
    if (xdr_bytes(&xdrs, &message, &size, size) == FALSE) {
      printf("ERROR in xdr conversion\n");
      xdr_destroy(&xdrs);
      return (ERROR);
    }
  }
 
  xdr_destroy(&xdrs);
  return (nbytes);
}




/*-----------------------------------------------------------------------
Name: WriteBytes(int sockfd, char *message, int size)
In: signature
Out: message to socket
Comments: Function to write message to a socket using XDR.
------------------------------------------------------------------------*/
int WriteBytes(int sockfd, char *message, int size) {

  int nbytes = 0;
  XDR xdrs;
  char *buf;

  buf = (char *)malloc(size+16);
  xdrmem_create(&xdrs, buf, size+16, XDR_ENCODE);
  if (xdr_bytes(&xdrs, &message, &size, size) == FALSE) {
    printf("ERROR in xdr conversion \n");
    xdr_destroy(&xdrs);
    return (ERROR);
  }
  if ((nbytes = writen(sockfd, buf, size)) != size) {
    xdr_destroy(&xdrs);
    return(ERROR);
  }

  xdr_destroy(&xdrs);
  return(nbytes);

}





/*-----------------------------------------------------------------------
Name: ReadShort(int sockfd, char *message, int size)
In: signature
Out: message from socket
Comments: Function to read message from a socket using XDR.
------------------------------------------------------------------------*/
int ReadShort(int sockfd, short *message, int size)
{
   XDR             xdrs;
   int             buf_size = size * sizeof(short);
   char            buf[buf_size];
   int             nbytes;

   xdrmem_create(&xdrs, buf, buf_size, XDR_DECODE);

   if ((nbytes = readn(sockfd, buf, buf_size)) != buf_size) {
     xdr_destroy(&xdrs);
     return (ERROR);
   } else {
     if (xdr_vector(&xdrs, message, size, sizeof(short), 
		    xdr_short) == FALSE) {
       printf("ERROR in xdr conversion\n");
       xdr_destroy(&xdrs);
       return (ERROR);
     }
   }
   xdr_destroy(&xdrs);

   return(nbytes / sizeof(short));
 }




/*-----------------------------------------------------------------------
Name: ReadBytes(int sockfd, char *message, int size)
In: signature
Out: message from socket
Comments: Function to read message from a socket using XDR.
------------------------------------------------------------------------*/
int ReadInt(int sockfd, int *message, int size)
{
   XDR             xdrs;
   int             buf_size = size * sizeof(int);
   char            buf[buf_size];
   int             nbytes;

   xdrmem_create(&xdrs, buf, buf_size, XDR_DECODE);

   if ((nbytes = readn(sockfd, buf, buf_size)) != buf_size) {
     xdr_destroy(&xdrs);
     return (ERROR);
   } else {
     if (xdr_vector(&xdrs, message, size, sizeof(int), 
		    xdr_int) == FALSE) {
       printf("ERROR in xdr conversion\n");
       xdr_destroy(&xdrs);
       return (ERROR);
     }
   }
   xdr_destroy(&xdrs);

   return(nbytes / sizeof(int));
 }




/*-----------------------------------------------------------------------
Name: ReadBytes(int sockfd, char *message, int size)
In: signature
Out: message from socket
Comments: Function to read message from a socket using XDR.
------------------------------------------------------------------------*/
int ReadShort2Float(int sockfd, float *message, int size) {

   XDR             xdrs;
   int             buf_size = size * sizeof(short);
   char            buf[buf_size];
   int             nbytes, i, elem_processed = 0, err_flag = FALSE;
   short           temp;

   xdrmem_create(&xdrs, buf, buf_size, XDR_DECODE);

   if ((nbytes = readn(sockfd, buf, buf_size)) != buf_size) {
     xdr_destroy(&xdrs);
     return (ERROR);
   } else {
     for (i = 0; i < size; i++) {
       elem_processed += elem_processed;
       if (xdr_short(&xdrs, &temp) == FALSE) {
	 printf("ERROR in xdr conversion\n");
	 xdr_destroy(&xdrs);
	 err_flag = TRUE;
	 break;
       }
       message[i] = (float)temp;
     }
     if (err_flag) return (ERROR);
   }
   
   xdr_destroy(&xdrs);
   return(elem_processed);       

}




/*-----------------------------------------------------------------------
Name: ReadBytes(int sockfd, char *message, int size)
In: signature
Out: message from socket
Comments: Function to read message from a socket using XDR.
------------------------------------------------------------------------ */
int ReadDouble(int sockfd, double *message, int size)
{
   XDR             xdrs;
   int             buf_size = size * sizeof(double);
   char            buf[buf_size];
   int             nbytes;

   xdrmem_create(&xdrs, buf, buf_size, XDR_DECODE);
   if ((nbytes = readn(sockfd, buf, buf_size)) != buf_size) {
     xdr_destroy(&xdrs);
     return (ERROR);
   } else {
     if (xdr_vector(&xdrs, message, size, sizeof(double), 
		    xdr_double) == FALSE) {
       printf("ERROR in xdr conversion\n");
       xdr_destroy(&xdrs);
       return (ERROR);
     }
   }
   xdr_destroy(&xdrs);
   return(nbytes / sizeof(double));

}




/*-----------------------------------------------------------------------
Name: WriteBytes(int sockfd, char *message, int size)
In: signature
Out: message to socket
Comments: Function to write message to a socket using XDR.
------------------------------------------------------------------------*/
int WriteShort(int sockfd, short *message, int size) {

  int nbytes = 0;
  XDR xdrs;
  int             buf_size = size * sizeof(short);
  char            buf[buf_size];

  xdrmem_create(&xdrs, buf, buf_size, XDR_ENCODE);
  if (xdr_vector(&xdrs, message, size, sizeof(short), xdr_short) == FALSE) {
    printf("ERROR in xdr conversion \n");
    xdr_destroy(&xdrs);
    return (ERROR);
  }

  if ((nbytes = writen(sockfd, buf, buf_size)) != buf_size) {
    xdr_destroy(&xdrs);
    return(ERROR);
  }

  xdr_destroy(&xdrs);
  return(nbytes/sizeof(short));

}





/*-----------------------------------------------------------------------
Name: WriteBytes(int sockfd, char *message, int size)
In: signature
Out: message to socket
Comments: Function to write message to a socket using XDR.
------------------------------------------------------------------------*/
int WriteInt(int sockfd, int *message, int size) {

  int   nbytes = 0;
  XDR   xdrs;
  int   buf_size = size * sizeof(int);
  char  buf[buf_size];

  xdrmem_create(&xdrs, buf, buf_size, XDR_ENCODE);
  if (xdr_vector(&xdrs, message, size, sizeof(int), xdr_int) == FALSE) {
    printf("ERROR in xdr conversion \n");
    xdr_destroy(&xdrs);
    return (ERROR);
  }

  if ((nbytes = writen(sockfd, buf, buf_size)) != buf_size) {
    xdr_destroy(&xdrs);
    return(ERROR);
  }

  xdr_destroy(&xdrs);
  return(nbytes/sizeof(int));

}





/*-----------------------------------------------------------------------
Name: WriteBytes(int sockfd, char *message, int size)
In: signature
Out: message to socket
Comments: Function to write message to a socket using XDR.
------------------------------------------------------------------------*/
int WriteDouble(int sockfd, double *message, int size) {

  int nbytes = 0;
  XDR xdrs;
  int             buf_size = size * sizeof(double);
  char            buf[buf_size];

  xdrmem_create(&xdrs, buf, buf_size, XDR_ENCODE);
  if (xdr_vector(&xdrs, message, size, sizeof(double), xdr_double) == FALSE) {
    printf("ERROR in xdr conversion \n");
    xdr_destroy(&xdrs);
    return (ERROR);
  }

  if ((nbytes = writen(sockfd, buf, buf_size)) != buf_size) {
    xdr_destroy(&xdrs);
    return(ERROR);
  }

  xdr_destroy(&xdrs);

  return(nbytes/sizeof(double));

}
